# required packages:
#  * pyenchant for English Speel Checker
#      usage SpellChecker.exe [portnumber (by default 9903]
#      reads from the indicated socket
#           - c[text]
#             check the spell of the indicated text
#               returns: OK if not spell error found
#                        comma separated list of the errors in pairs (word, position)
#                   e.g.: "sme text is betwen one and 100 words"
#                       returns: sme,0,betwen,12
#
#           - s[word]
#             give the suggestions (max 5 elements) for the indicated word
#                   e.g: sbetween
#                       returns: between,betwixt,betoken,betaken,Bethany
#                             or OK if not suggestions 
#
#           - a[word]
#             appends the indicated word to the dictionary
#               returns: OK or KO if ther are more than one word
#               * the personal dict is stored in "c:\users\{user}\AppData\roaming\enchant\en.dic"
#
#           - #exit#
#               to stop listening and close the communication
#               returns OK
#
#           - any received message that not start with "c", "s" or "a" is responded as KO (unrecognized command)
#
#  * py2exe for create distribuible external executable
#       required setup.py file
#       run cmd on "C:\python27" "python setup.py py2exe"


import socket
import sys
import enchant
from enchant.checker import SpellChecker
import re
import itertools


# port argument management
PortNumber = 9903
if len(sys.argv) > 1:
    try:
        PortNumber = int(sys.argv[1])
    except:
        pass


# socket comm reading management
EndCommSymbol = "\b"
def socket_receive(sock):
    total_data=[]
    data=''
    while True:
            data=sock.recv(8192)
            if EndCommSymbol in data:
                total_data.append(data[:data.find(EndCommSymbol)])
                break
            total_data.append(data)
            if len(total_data)>1:
                last_pair=total_data[-2]+total_data[-1]
                if EndCommSymbol in last_pair:
                    total_data[-2]=last_pair[:last_pair.find(EndCommSymbol)]
                    total_data.pop()
                    break
    return ''.join(total_data)






#### open the communications 
sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
try:
    sock.bind(('localhost', PortNumber))
except socket.error as msg:
    print 'Bind failed. Error Code : ' + str(msg[0]) + ' Message ' + msg[1]
    sys.exit()
     
print "Listening at port: " + str(PortNumber)
sock.listen(1)

checker = SpellChecker("en")  # english spell checker

regexp = re.compile(r'([a-zA-Z]+[-\/\\]?[0-9]+|[0-9]+[-\/\\]?[a-zA-Z]+)') # to detect words with numbers in it (due to are not identified as spelling error by the checker)

while True:

    #### wait the petitions
    conn, addr = sock.accept()
    data = socket_receive(conn)
    print "received: " + data

    #### process the command
    resp = "OK"
    if data[0] == 'c': ## check the spell of any word in the sentence
        text = data[1:]
        checker.set_text(text)
        res = []
        for err in checker:
            res.append([err.word, err.wordpos])

        # detect words with numbers in it (due to are not identified as spelling error by the checker)
        for match in regexp.finditer(text):
            res.append([ text[match.start():text.find(" ", match.start())] , match.start()])

        if len(res) > 0:
            res.sort()
            print list(res for res,_ in itertools.groupby(res))
            
            resp = ""
            for r in res:
                resp = resp + str(r[0]) + "," + str(r[1]) + ","
            resp = resp[:-1]

    else: ## suggestions
        if data[0] == 's':
            res = checker.suggest(data[1:])[0:5]
            if len(res) > 0:
                resp = ""
                for r in res:
                    resp = resp + r + ","
                resp = resp[:-1]
           
        else: ## append new words
            if data[0] == 'a':
                if  data[1:].find(" ") > -1: # only ONE word is allowed
                    resp = "KO"
                else:
                    checker.add(data[1:])
        
            else:
                if data != "#echo#" and data != "#exit#": ## unrecognized order
                    resp = "KO"


    #### return the results
    print "responded: " + resp
    conn.sendall(resp + EndCommSymbol)
    print

    if data == "#exit#":
        break


   
sock.close()


